package core.api.controller;

import core.api.models.auth.SimpleStatusResponse;
import core.api.models.timemanager.taskgroup.RecursiveTaskgroupInfo;
import core.api.models.timemanager.taskgroup.TaskgroupDetailInfo;
import core.api.models.timemanager.taskgroup.TaskgroupEntityInfo;
import core.api.models.timemanager.taskgroup.TaskgroupFieldInfo;
import core.entities.timemanager.Taskgroup;
import core.services.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;
import java.util.List;
import java.util.Set;

@CrossOrigin(origins = "*", maxAge = 3600)
@RestController
@RequestMapping("/api")
public class TaskgroupController {

    private final TaskgroupService taskgroupService;
    private final TaskService taskService;

    public TaskgroupController(@Autowired TaskgroupService taskgroupService, @Autowired TaskService taskService) {
        this.taskgroupService = taskgroupService;
        this.taskService = taskService;
    }

    @PutMapping("/taskgroups")
    public ResponseEntity<?> createTaskgroup(@Valid @RequestBody TaskgroupFieldInfo fieldInfo) {
        String username = SecurityContextHolder.getContext().getAuthentication().getName();
        ServiceResult<Taskgroup> creationResult = taskgroupService.addTaskgroup(fieldInfo, username);
        if(creationResult.getExitCode() == ServiceExitCode.OK) {
            return ResponseEntity.ok(new TaskgroupEntityInfo(creationResult.getResult()));
        } else {
            return ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
        }
    }

    @PostMapping("/taskgroups/{taskgroupID}")
    public ResponseEntity<?> editTaskgroup(@PathVariable long taskgroupID, @Valid @RequestBody TaskgroupFieldInfo taskgroupFieldInfo) {
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!taskgroupPermissionResult.isHasPermissions()) {
           return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        ServiceExitCode serviceExitCode = taskgroupService.editTaskgroup(taskgroupPermissionResult.getResult(), taskgroupFieldInfo);
        if(serviceExitCode == ServiceExitCode.OK) {
            return ResponseEntity.ok(new SimpleStatusResponse("success"));
        } else {
            return ResponseEntity.status(409).body(new SimpleStatusResponse("failed"));
        }
    }

    @DeleteMapping("/taskgroups/{taskgroupID}")
    public ResponseEntity<?> deleteTaskgroup(@PathVariable long taskgroupID) {
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if (!taskgroupPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        taskgroupService.deleteTaskgroup(taskgroupPermissionResult.getResult());
        return ResponseEntity.ok(new SimpleStatusResponse("success"));
    }

    @GetMapping("/taskgroups")
    public ResponseEntity<List<TaskgroupEntityInfo>> listTaskgroupsOfUser() {
        List<Taskgroup> taskgroups = taskgroupService.getTopTaskgroupsByUser(SecurityContextHolder.getContext().getAuthentication().getName());
        List<TaskgroupEntityInfo> taskgroupEntityInfos = taskgroups.stream().map(TaskgroupEntityInfo::new).toList();
        return ResponseEntity.ok(taskgroupEntityInfos);
    }

    @GetMapping("/taskgroups/tree")
    public ResponseEntity<List<RecursiveTaskgroupInfo>> listTaskgroupsOfUserAsTree() {
        List<Taskgroup> taskgroups = taskgroupService.getTopTaskgroupsByUser(SecurityContextHolder.getContext().getAuthentication().getName());
        List<RecursiveTaskgroupInfo> taskgroupEntityInfos = taskgroups.stream().map(RecursiveTaskgroupInfo::new).toList();
        return ResponseEntity.ok(taskgroupEntityInfos);
    }

    @GetMapping("/taskgroups/{taskgroupID}")
    public ResponseEntity<?> getDetails(@PathVariable long taskgroupID) {
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if(!taskgroupPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if(taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        return ResponseEntity.ok(new TaskgroupDetailInfo(taskgroupPermissionResult.getResult()));
    }

    @DeleteMapping("/taskgroups/{taskgroupID}/clear")
    public ResponseEntity<?> clearTasks(@PathVariable long taskgroupID) {
        PermissionResult<Taskgroup> taskgroupPermissionResult = taskgroupService.getTaskgroupByIDAndUsername(taskgroupID, SecurityContextHolder.getContext().getAuthentication().getName());
        if (!taskgroupPermissionResult.isHasPermissions()) {
            return ResponseEntity.status(403).body(new SimpleStatusResponse("failed"));
        }

        if (taskgroupPermissionResult.getExitCode() == ServiceExitCode.MISSING_ENTITY) {
            return ResponseEntity.status(404).body(new SimpleStatusResponse("failed"));
        }

        taskService.clearTasks(taskgroupPermissionResult.getResult());
        return ResponseEntity.ok(new SimpleStatusResponse("success"));
    }
}
